package com.sbh.background_locations.services

import android.content.Context
import android.location.Location
import android.util.Log
import com.google.firebase.FirebaseApp
import com.google.firebase.FirebaseOptions
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.database.FirebaseDatabase
import com.sbh.background_locations.adapters.SBHConfig
import com.sbh.background_locations.adapters.SBHDatabaseRefPath
import com.sbh.background_locations.adapters.SBHFirebaseConfig
import com.sbh.background_locations.utils.SingletonHolder

class FirebaseDatabaseService(context: Context) {
    private var mFirebaseConfig : SBHFirebaseConfig? = null
    private lateinit var app: FirebaseApp
    private lateinit var auth: FirebaseAuth
    private lateinit var database: FirebaseDatabase

    companion object : SingletonHolder<FirebaseDatabaseService, Context>(::FirebaseDatabaseService) {
        private const val TAG = "FirebaseDatabaseService"
    }

    init {
        initialize(context)
    }
    
    internal fun initialize(context: Context) {
        mFirebaseConfig = SBHConfig.getInstance(context).firebaseConfig()
        
        if(mFirebaseConfig == null)
            return
        
        var hasBeenInitialized = false
        if(mFirebaseConfig?.appName != null) {
            val fbsLcl = FirebaseApp.getApps(context)
            for (app in fbsLcl) {
                if (app.name == mFirebaseConfig!!.appName) {
                    hasBeenInitialized = true
                }
            }
        }

        Log.d(TAG, hasBeenInitialized.toString())

        val options = FirebaseOptions.Builder()
                .setApplicationId(mFirebaseConfig?.applicationId!!) // Required for Analytics.
                .setApiKey(mFirebaseConfig!!.apiKey!!) // Required for Auth.
                .setDatabaseUrl(mFirebaseConfig!!.databaseUrl!!) // Required for RTDB.
                .build()

        if(!hasBeenInitialized) {
            if (mFirebaseConfig!!.appName == null)
                FirebaseApp.initializeApp(context, options)
            else
                FirebaseApp.initializeApp(context, options, mFirebaseConfig!!.appName!!)
        }

        app = FirebaseApp.getInstance(mFirebaseConfig!!.appName!!)
        auth = FirebaseAuth.getInstance(app)
        database = FirebaseDatabase.getInstance(app)
    }

    fun updateLocation(location: Location) {
        if(::app.isInitialized && ::auth.isInitialized && ::database.isInitialized) {
            if (auth.currentUser != null) {
                Log.d(TAG, "UPDATING LOCATION")
                writeToDb(location)
            } else {
                Log.d(TAG, "UPDATING LOCATION - USER NOT EXISTS")
                auth.signInWithEmailAndPassword(mFirebaseConfig?.email!!, mFirebaseConfig!!.password!!)
                        .addOnCompleteListener { task ->
                            run {
                                if (task.isSuccessful) {
                                    Log.d(TAG, "signInWithCustomToken:success")
                                    writeToDb(location)
                                } else {
                                    Log.w(TAG, "signInWithCustomToken:failure", task.exception)
                                }
                            }
                        }
            }
        }
    }

    private fun writeToDb(location: Location) {
        for(ref in mFirebaseConfig?.databaseRefPaths!!) {
            if(ref.method == SBHDatabaseRefPath.UPDATE_METHOD_VALUE) {
                database.reference.child(ref.path).setValue(location)
            } else if(ref.method == SBHDatabaseRefPath.UPDATE_METHOD_ADD) {
                database.reference.child(ref.path).push().setValue(location)
            }
        }
    }
}
